// Copyright 2002 - 2008, 2010, 2011 National Technology Engineering
// Solutions of Sandia, LLC (NTESS). Under the terms of Contract
// DE-NA0003525 with NTESS, the U.S. Government retains certain rights
// in this software.
//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
 // 
 //     * Redistributions of source code must retain the above copyright
 //       notice, this list of conditions and the following disclaimer.
 // 
 //     * Redistributions in binary form must reproduce the above
 //       copyright notice, this list of conditions and the following
 //       disclaimer in the documentation and/or other materials provided
 //       with the distribution.
 // 
//     * Neither the name of NTESS nor the names of its contributors
//       may be used to endorse or promote products derived from this
//       software without specific prior written permission.
//
 // THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 // "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 // LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 // A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 // OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 // SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 // LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 // DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 // THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef STK_NGP_NGPSPACES_H_
#define STK_NGP_NGPSPACES_H_

#include <Kokkos_Core.hpp>

#ifdef KOKKOS_ENABLE_OPENMP
#include <Kokkos_OpenMP.hpp>
#endif

namespace ngp {

#ifdef KOKKOS_ENABLE_CUDA
  typedef Kokkos::Cuda     ExecSpace ;
#elif defined(KOKKOS_ENABLE_OPENMP)
  typedef Kokkos::OpenMP   ExecSpace ;
#else
  typedef Kokkos::Serial   ExecSpace ;
#endif

#ifdef KOKKOS_ENABLE_CUDA
  typedef Kokkos::Serial   HostExecSpace ;
#elif defined(KOKKOS_ENABLE_OPENMP)
  typedef Kokkos::OpenMP   HostExecSpace ;
#else
  typedef Kokkos::Serial   HostExecSpace ;
#endif

#ifdef KOKKOS_ENABLE_CUDA
   typedef Kokkos::CudaSpace    MemSpace;
#elif defined(KOKKOS_ENABLE_OPENMP)
   typedef Kokkos::OpenMP       MemSpace;
#else
   typedef Kokkos::HostSpace    MemSpace;
#endif

#ifdef KOKKOS_ENABLE_CUDA
typedef Kokkos::CudaUVMSpace UVMMemSpace;
#elif defined(KOKKOS_ENABLE_OPENMP)
typedef Kokkos::OpenMP       UVMMemSpace;
#else
typedef Kokkos::HostSpace    UVMMemSpace;
#endif

typedef Kokkos::Schedule<Kokkos::Dynamic> ScheduleType;

} // namespace ngp

#endif /* STK_NGP_NGPSPACES_H_ */
